//Initialize vars
local isForest = 0
local wasFalling = 0
local wasGrounded = 0
local wasUnderwater = 0
local wasOxygen = 0
local wasSuper = 0
local wasCarried = 0
local prevSlope = 0
local boosted = 0



//Ring effect object (from Rebound Dash by Krabs)
freeslot(
	"SPR_REBO",
	"MT_REBOUND",
	"S_REBOUND"
)
mobjinfo[MT_REBOUND] = {
	doomednum = -1,
	spawnstate = S_REBOUND,
	flags = MF_NOCLIP|MF_SCENERY|MF_NOGRAVITY
}
states[S_REBOUND] = {
	sprite = SPR_REBO,
	frame = TR_TRANS10|FF_PAPERSPRITE|A
}



//Make it so you can't avoid water slowdown by jumping
addHook("JumpSpecial", function(player)
	if (player.mo.skin == "forest")
	and P_IsObjectOnGround(player.mo)
	and (FixedHypot(player.mo.momx, player.mo.momy) >= player.runspeed)
	and (player.cmd.forwardmove or player.cmd.sidemove)
	and FixedHypot(player.mo.momx, player.mo.momy) < 120*FRACUNIT
	and (player.mo.eflags & MFE_TOUCHWATER)
	and not (player.mo.eflags & MFE_UNDERWATER)
		P_Thrust(player.mo, R_PointToAngle2(0, 0, player.mo.momx, player.mo.momy), -10*FRACUNIT)
	end
end,MT_PLAYER)



//Replacing vanilla with custom music only for Forest
addHook("MusicChange", function(oldname, newname)
	if isForest
		if newname == "_super"
			return "fo_super"
		end
	end
end)



//Start the game with 0 continues
addHook("MapLoad", function(mapnum)
	if isForest and (mapnum == spstage_start) then consoleplayer.continues = 0 end
end)



//Function used for pesky P_MobjFlip comparisons
rawset(_G,"MobjFlipCompare",function(mo, comparison, greaterThan)
	if greaterThan
		if  (P_MobjFlip(mo) == 1
		and mo.momz > comparison)
		or (P_MobjFlip(mo) == -1
		and mo.momz < comparison * -1)
			return true
		else
			return false
		end
	else
		if  (P_MobjFlip(mo) == 1
		and mo.momz < comparison)
		or (P_MobjFlip(mo) == -1
		and mo.momz > comparison * -1)
			return true
		else
			return false
		end
	end
end)



//Forest tech trails
rawset(_G,"P_SpawnTechTrail",function(player, shieldColor)
	local thokMobj = P_SpawnMobjFromMobj(player.mo, 0, 0, 0, MT_THOK)
	thokMobj.color = SKINCOLOR_FLAME
	if player.powers[pw_super]
		thokMobj.blendmode = AST_ADD
		if not (leveltime % (TICRATE/16))
			P_SpawnMobjFromMobj(player.mo, 0, 0, 8*FRACUNIT, MT_SUPERSPARK)
		end
	else
		local ghost = P_SpawnGhostMobj(player.mo)
		ghost.color = SKINCOLOR_MIDNIGHT
		if (leveltime & 1)
			ghost.frame = $&~FF_TRANSMASK
			ghost.frame = $|tr_trans10<<FF_TRANSSHIFT
		end
		if shieldColor
			if (player.powers[pw_shield] & SH_WHIRLWIND)
				if (player.powers[pw_shield] & SH_PROTECTELECTRIC)
					thokMobj.color = SKINCOLOR_ICY
				else
					thokMobj.color = SKINCOLOR_CLOUDY
				end
			elseif (player.powers[pw_shield] & SH_PROTECTFIRE)
				thokMobj.color = SKINCOLOR_PEPPER
			elseif (player.powers[pw_shield] & SH_PROTECTELECTRIC)
				thokMobj.color = SKINCOLOR_YELLOW
			elseif (player.powers[pw_shield] & SH_FORCE)
				if (player.powers[pw_shield] & SH_FORCEHP)
					thokMobj.color = SKINCOLOR_MAGENTA
				else
					thokMobj.color = SKINCOLOR_PURPLE
				end
				
			end
		end
	end
	thokMobj.renderflags = $|RF_FULLBRIGHT|RF_NOCOLORMAPS
	thokMobj.spriteyscale = player.mo.spriteyscale + FRACUNIT/4
	thokMobj.spritexscale = player.mo.spritexscale + FRACUNIT/4
	thokMobj.spriteyoffset = -32*thokMobj.spriteyscale + 40*FRACUNIT
end)



//Fixing stuff after respawning
addHook("PlayerSpawn", function(player)
	if player.mo.skin == "forest"
		wasOxygen = 20*TICRATE
		wasSuper = 0
	end
end, MT_PLAYER)



addHook("PlayerThink", function(player)
	if player.mo.skin == "forest"
		isForest = 1
		
		//Initializing relevant player vars
		if player.sneakerMul == nil
			player.sneakerMul = 1
			player.prevmomz = 0
			player.zWaterMul = FRACUNIT
			player.wasJustlaunched = 0
		end
		
		//Setting player vars
		if player.powers[pw_sneakers]
			player.sneakerMul = 2
		else
			player.sneakerMul = 1
		end
		player.prevmomz = player.mo.momz
		if player.mo.eflags & MFE_UNDERWATER
			player.zWaterMul = FRACUNIT*117/200
		else
			player.zWaterMul = FRACUNIT
		end
		
		//Shorthanding
		local moDir = R_PointToAngle2(0, 0, player.mo.momx, player.mo.momy)
		
		
		//Super glows
		if player.powers[pw_super]
			player.mo.renderflags = $|RF_NOCOLORMAPS
			if not player.mo.superTrail1 or not player.mo.superTrail1.valid// or (player.mo.superTrail1.fuse == 0)
				player.mo.superTrail1 = P_SpawnGhostMobj(player.mo)
				//player.mo.superTrail1.momx = 2*(P_RandomFixed()-FRACUNIT/2)
				//player.mo.superTrail1.momy = 2*(P_RandomFixed()-FRACUNIT/2)
				player.mo.superTrail1.momz = 4*(P_RandomFixed())
				player.mo.superTrail1.fuse = P_RandomRange(1, 4)
				player.mo.superTrail1.colorized = true
				player.mo.superTrail1.color = SKINCOLOR_FLAME
				player.mo.superTrail1.renderflags = $|RF_FULLBRIGHT|RF_NOCOLORMAPS
				player.mo.superTrail1.blendmode = AST_ADD
				player.mo.superTrail1.scale = FRACUNIT*11/10
			end
			
			if not player.mo.superTrail2 or not player.mo.superTrail2.valid// or (player.mo.superTrail2.fuse == 0)
				player.mo.superTrail2 = P_SpawnGhostMobj(player.mo)
				//player.mo.superTrail2.momx = 2*(P_RandomFixed()-FRACUNIT/2)
				//player.mo.superTrail2.momy = 2*(P_RandomFixed()-FRACUNIT/2)
				player.mo.superTrail2.momz = 4*(P_RandomFixed())
				player.mo.superTrail2.fuse = P_RandomRange(1, 4)
				player.mo.superTrail2.colorized = true
				player.mo.superTrail2.color = SKINCOLOR_FLAME
				player.mo.superTrail2.renderflags = $|RF_FULLBRIGHT|RF_NOCOLORMAPS
				player.mo.superTrail2.blendmode = AST_ADD
				player.mo.superTrail2.scale = FRACUNIT*11/10
			end
			
			if not player.mo.superTrail3 or not player.mo.superTrail3.valid// or (player.mo.superTrail3.fuse == 0)
				player.mo.superTrail3 = P_SpawnGhostMobj(player.mo)
				//player.mo.superTrail3.momx = 2*(P_RandomFixed()-FRACUNIT/2)
				//player.mo.superTrail3.momy = 2*(P_RandomFixed()-FRACUNIT/2)
				player.mo.superTrail3.momz = 4*(P_RandomFixed())
				player.mo.superTrail3.fuse = P_RandomRange(1, 4)
				player.mo.superTrail3.colorized = true
				player.mo.superTrail3.color = SKINCOLOR_FLAME
				player.mo.superTrail3.renderflags = $|RF_FULLBRIGHT|RF_NOCOLORMAPS
				player.mo.superTrail3.blendmode = AST_ADD
				player.mo.superTrail3.scale = FRACUNIT*11/10
			end
			
			/*if player.mo.superTrail1 and player.mo.superTrail1.valid
				print(player.mo.superTrail1.fuse)
				if (player.mo.superTrail1.fuse == 0)
					player.mo.superTrail1 = nil
					print(1)
				end
			end
			if player.mo.superTrail2 and player.mo.superTrail2.valid and player.mo.superTrail2.fuse == 0
				player.mo.superTrail2 = nil
				print(2)
			end
			if player.mo.superTrail3 and player.mo.superTrail3.valid and player.mo.superTrail3.fuse == 0
				player.mo.superTrail3 = nil
				print(3)
			end*/
			
			
			if player.mo.superTrail1 and player.mo.superTrail1.valid
			and player.mo.superTrail2 and player.mo.superTrail2.valid
			and player.mo.superTrail3 and player.mo.superTrail3.valid
				if player.speed == 0
					player.mo.superTrail1.drawangle = player.drawangle
					player.mo.superTrail2.drawangle = player.drawangle
					player.mo.superTrail3.drawangle = player.drawangle
				end
				if (leveltime & 1)
					player.mo.superTrail1.frame = $&~FF_TRANSMASK
					player.mo.superTrail1.frame = $|tr_trans10<<FF_TRANSSHIFT
					player.mo.superTrail2.frame = $&~FF_TRANSMASK
					player.mo.superTrail2.frame = $|tr_trans10<<FF_TRANSSHIFT
					player.mo.superTrail3.frame = $&~FF_TRANSMASK
					player.mo.superTrail3.frame = $|tr_trans10<<FF_TRANSSHIFT
				end
				P_InstaThrust(player.mo.superTrail1, moDir, FixedHypot(player.mo.momx, player.mo.momy))
				P_InstaThrust(player.mo.superTrail2, moDir, FixedHypot(player.mo.momx, player.mo.momy))
				P_InstaThrust(player.mo.superTrail3, moDir, FixedHypot(player.mo.momx, player.mo.momy))
				//P_SetOrigin(player.mo.superTrail1, player.mo.x, player.mo.y, player.mo.superTrail1.z)
				//P_SetOrigin(player.mo.superTrail2, player.mo.x, player.mo.y, player.mo.superTrail2.z)
				//P_SetOrigin(player.mo.superTrail3, player.mo.x, player.mo.y, player.mo.superTrail3.z)
			end
		else
			player.mo.renderflags = $&~RF_NOCOLORMAPS
		end
		//Transforming
		if player.mo.state == S_PLAY_SUPER_TRANS5
			player.pflags = $&~PF_THOKKED
			P_StartQuake(20*FRACUNIT, TICRATE/4)
			//S_StartSound(player.mo, sfx_kc3b)
			S_StartSound(player.mo, sfx_litng1)
		end
		//Detransforming
		if not player.powers[pw_super]
		and wasSuper
			S_StartSound(player.mo, sfx_s3k74)
		end
		
		
		//Make it so he sucks at breathing
		if player.powers[pw_underwater] > 20*TICRATE
			player.powers[pw_underwater] = 20*TICRATE
		end
		//Aquaphobia noise is much faster
		if player.powers[pw_underwater] > 11*TICRATE
		and not ((player.powers[pw_underwater] - TICRATE) % (2*TICRATE))
			S_StartSound(player.mo, sfx_wtrdng, player)
		end
		//Make it so it shows he sucks at breathing
		if not (player.mo.eflags & MFE_UNDERWATER)
		and wasUnderwater
		and wasOxygen < 19*TICRATE
		and not (player.powers[pw_shield] & SH_PROTECTWATER)
		and not P_PlayerInPain(player)
		and player.mo.health
			player.mo.state = S_PLAY_GASP
			if not S_SoundPlaying(player.mo, sfx_cdfm29)
				S_StartSound(player.mo, sfx_cdfm29)
			end
		end
		//Fixing sprite after sucking at breathing
		if (player.pflags & PF_JUMPED)
		and player.mo.state != S_PLAY_GASP
		and player.mo.state != S_PLAY_JUMP
		and not ((player.mo.state >= S_PLAY_SUPER_TRANS1 and player.mo.state <= S_PLAY_SUPER_TRANS6)
		or (player.mo.state >= S_PLAY_NIGHTS_TRANS1 and player.mo.state <= S_PLAY_NIGHTS_ATTACK)
		or (player.powers[pw_super] and player.dashing))
			player.mo.state = S_PLAY_JUMP
		end
		
		
		//Slide against walls (so momentum doesn't suddenly stop)
		player.mo.flags = $|MF_SLIDEME
		
		//9 Line (technically 4 Line) Momentum by kirb. Props to him!
		if (((FixedHypot(player.mo.momx, player.mo.momy) >= player.runspeed) and not (player.powers[pw_super] or player.powers[pw_sneakers]))
		or ((FixedHypot(player.mo.momx, player.mo.momy) >= 2*player.runspeed) and (player.powers[pw_super] or player.powers[pw_sneakers])))
		and (player.cmd.forwardmove or player.cmd.sidemove)
		and FixedHypot(player.mo.momx, player.mo.momy) < 120*FRACUNIT
			if P_IsObjectOnGround(player.mo)
			and (player.mo.eflags & MFE_TOUCHWATER)
			and not (player.mo.eflags & MFE_UNDERWATER)
			and not player.powers[pw_super]
				player.mo.friction = FRACUNIT*255/256
			else
				player.mo.friction = FRACUNIT
			end
		end
		//Speed cap
		if FixedHypot(player.mo.momx, player.mo.momy) > 120*FRACUNIT
			P_InstaThrust(player.mo, moDir, 120*FRACUNIT)
		end
		//Underwater speed cap
		if (player.mo.eflags & MFE_UNDERWATER)
		and FixedHypot(player.mo.momx, player.mo.momy) > 22*FRACUNIT * player.sneakerMul
			P_Thrust(player.mo, R_PointToAngle2(0, 0, player.mo.momx, player.mo.momy), -FRACUNIT/2)
		end
		
		
		//Allowing you to thok again after going on a spring among other things
		if ((player.mo.eflags & MFE_SPRUNG)
		or (wasCarried and not player.powers[pw_carry]))
		and not P_IsObjectOnGround(player.mo)
			player.charflags = $|SF_NOJUMPSPIN|SF_NOJUMPDAMAGE
			player.pflags = $|PF_JUMPED
		end
		//Changing sprite when falling off of the ground (and allowing you to thok)
		if not P_IsObjectOnGround(player.mo)
		and wasGrounded
		and not P_PlayerInPain(player)
		and player.mo.health
			player.pflags = $|PF_JUMPED
			if player.pflags & PF_SPINNING
				player.mo.state = S_PLAY_ROLL
			else
				if player.mo.momz * P_MobjFlip(player.mo) > 0
					player.mo.state = S_PLAY_SPRING
				else
					player.mo.state = S_PLAY_FALL
				end
			end
		end
		
		
		//Spawning afterimages to indicate when the player is in "boost mode"
		//Also sonic boom when entering the mode
		if FixedHypot(player.mo.momx, player.mo.momy) > 50*FRACUNIT
			if not boosted
				boosted = 1
				P_StartQuake(20*FRACUNIT, TICRATE/4)
				S_StartSound(player.mo, sfx_kc3b)
				local circle = P_SpawnMobjFromMobj(player.mo, 0, 0, player.mo.scale * 24, MT_REBOUND)
				circle.angle = player.mo.angle + ANGLE_90
				circle.fuse = 7
				circle.scale = FRACUNIT / 3
				circle.destscale = 10*FRACUNIT
				circle.colorized = true
				circle.color = SKINCOLOR_FLAME
				circle.momx = -player.mo.momx / 2
				circle.momy = -player.mo.momy / 2
				if FixedHypot(player.mo.momx, player.mo.momy) < 55*FRACUNIT
					if (player.powers[pw_super] or player.powers[pw_sneakers])
						P_InstaThrust(player.mo, moDir, 70*FRACUNIT)
					else
						P_InstaThrust(player.mo, moDir, 60*FRACUNIT)
					end
				end
			end
			//P_StartQuake(2*FRACUNIT, TICRATE/4)
			player.powers[pw_strong] = $|STR_SPIKE
			player.charflags = $|SF_RUNONWATER
			local ghost = P_SpawnGhostMobj(player.mo)
			ghost.fuse = 3
		else
			boosted = 0
			player.powers[pw_strong] = $&~STR_SPIKE
			player.charflags = $&~SF_RUNONWATER
		end
		
		//Special case water running (runonwater when super and not when pressing spin)
		if player.powers[pw_super]
			player.charflags = $|SF_RUNONWATER
		end
		if (player.cmd.buttons & BT_SPIN)
		or player.stomping
			player.charflags = $&~SF_RUNONWATER
		end
		
		
		//Changed slope physics (makes slopes affect you more going straight and backward)
		if not (player.mo.standingslope == nil)
		and FixedHypot(player.mo.momx, player.mo.momy) > 0
			
			//Normal slope physics
			if not (player.pflags & PF_SPINNING)
				
				//Check for no sliding while not moving
				if (player.cmd.forwardmove or player.cmd.sidemove)
				
					//Negating slope
					P_Thrust(player.mo, player.mo.standingslope.xydirection, FixedMul(player.mo.standingslope.zangle/(FRACUNIT/6), FRACUNIT/-32768*P_GetMobjGravity(player.mo)))
					//Normal negations have player.mo.standingslope.zangle/(FRACUNIT/6) times the rest of the equation. Decrease 6 for more influence and vice versa (for opposite dir)
				end
					
				//Thrust shift
				if FixedMul(player.mo.standingslope.zangle, cos(moDir - player.mo.standingslope.xydirection)) < 0
					P_Thrust(player.mo, moDir, FixedMul(AngleFixed(abs(player.mo.standingslope.zangle)), abs(cos(moDir - player.mo.standingslope.xydirection)))/32768*abs(P_GetMobjGravity(player.mo))/40)
				end
				//Normal is /40. Decrease for more slope influence and vice versa
				
			//Rolling slope physics
			else
				P_Thrust(player.mo, player.mo.standingslope.xydirection - ANGLE_180, FixedMul(player.mo.standingslope.zangle/(FRACUNIT/18), FRACUNIT/-32768*P_GetMobjGravity(player.mo)))
				//Increase 18 (in FRACUNIT/18) for more influence and vice versa
			end
		end
		//Little thing to stop weird glitching at the top of slopes
		if player.powers[pw_justlaunched] == 1
		and FixedHypot(player.mo.momx, player.mo.momy) < 5*FRACUNIT
		and not (player.cmd.buttons & BT_JUMP)
		and not ((player.cmd.buttons & BT_CUSTOM1) and (player.mo.eflags & MFE_JUSTHITFLOOR))
			P_InstaThrust(player.mo, prevSlope.xydirection - ANGLE_180, 5*FRACUNIT)
			P_SetObjectMomZ(player.mo, 2*FRACUNIT, true)
		end
		
		
		//Underwater previous frame?
		if (player.mo.eflags & MFE_UNDERWATER)
			wasUnderwater = 1
		else
			wasUnderwater = 0
		end
		//Water timer previous frame?
		wasOxygen = player.powers[pw_underwater]
		//On ground previous frame?
		if P_IsObjectOnGround(player.mo)
			wasGrounded = 1
		else
			wasGrounded = 0
		end
		//Falling previous frame?
		if player.mo.momz * P_MobjFlip(player.mo) <= 0
			wasFalling = 1
		else
			wasFalling = 0
		end
		if player.powers[pw_justlaunched]
			player.wasJustlaunched = 1
		else
			player.wasJustlaunched = 0
		end
		if player.powers[pw_super]
			wasSuper = 1
		else
			wasSuper = 0
		end
		if player.powers[pw_carry]
			wasCarried = 1
		else
			wasCarried = 0
		end
		prevSlope = player.mo.standingslope
		
	else
		isForest = 0
	end
end, MT_PLAYER)